@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- mode:python; tab-width: 2; coding: utf-8 -*-
 
 """
@@ -7,8 +6,7 @@ processors
 
 from __future__ import absolute_import
 
-__author__ = "caelum - http://caelum.com.br"
-__modified_by__  = "Carlos Martin <cmartin@liberalia.net>"
+__author__  = "Carlos Martin <cmartin@liberalia.net>"
 __license__ = "See LICENSE.restfulie for details"
 
 # Import here any required modules.
@@ -20,11 +18,11 @@ __all__ = [
     'ExecuteRequestProcessor',
     'PayloadMarshallingProcessor',
     'RedirectProcessor',
-    ]
+]
 
 # Project requirements
 from tornado.httputil import url_concat
-from tornado.httpclient import HTTPClient, AsyncHTTPClient
+from tornado.httpclient import HTTPClient, AsyncHTTPClient, HTTPError
 
 # local submodule requirements
 from .converters import Converters
@@ -69,7 +67,16 @@ class AuthenticationSyncProcessor(AuthenticationProcessor):
     def execute(self, callback, chain, request, env):
         method = AuthenticationProcessor.execute
         method, cred = method(self, callback, chain, request, env)
-        method and self.backends[method].authorize_sync(cred, request, env)
+
+        # Becouse authorize_sync calls are 'embbedded' on flow, we
+        # need a way to notify to callback that an error happened. To
+        # do that, we follow comvention to embbed error on a Response
+        # instance and pass it to callback. pylint: disable-msg=W0106
+        try:
+            method and self.backends[method].authorize_sync(cred, request, env)
+        except HTTPError, err:
+            callback(Response(err))
+            return
         # follow if no method used
         return chain.follow(callback, request, env)
 
@@ -82,15 +89,16 @@ class AuthenticationAsyncProcessor(AuthenticationProcessor):
         method = AuthenticationProcessor.execute
         method, cred = method(self, callback, chain, request, env)
         self.backends[method].authorize(cred, request, env, callback)
-        
+
+
 class MetaAuth(type):
     """Auth Metaclass"""
 
-    def __init__(mcs, name, bases, dct):
-        type.__init__(mcs, name, bases, dct)
+    def __init__(cls, name, bases, dct):
+        type.__init__(cls, name, bases, dct)
         if name.endswith("Auth"):
-            implements = mcs.implements
-            AuthenticationProcessor.backends[implements] = mcs()
+            implements = cls.implements
+            AuthenticationProcessor.backends[implements] = cls()
 
 
 #pylint: disable-msg=R0921
@@ -117,31 +125,56 @@ class ExecuteRequestProcessor(RequestProcessor):
     @staticmethod
     def _sync(request, env):
         """Run blocked"""
-        response = HTTPClient().fetch(
-            url_concat(request.uri, env["params"]),
-            method=request.verb,
-            headers=request.headers,
-            body=env.get("body"),
-            use_gzip = request.use_gzip,
-            progress_callback = request.progress_callback,
-            connect_timeout = request.connect_timeout,
-            request_timeout = request.request_timeout)
+        try:
+            response = HTTPClient().fetch(
+                url_concat(request.uri, env["params"]),
+                method=request.verb,
+                headers=request.headers,
+                body=env.get("body"),
+                use_gzip=request.use_gzip,
+                ca_certs=request.ca_certs,
+                progress_callback=request.progress_callback,
+                connect_timeout=request.connect_timeout,
+                request_timeout=request.request_timeout)
+        except Exception, err:
+            if not isinstance(err, HTTPError):
+                err = HTTPError(599, repr(err))
+            raise err
+        # All ok. return retval
         return Response(response)
 
     @staticmethod
     def _async(callback, request, env):
         """Run async"""
+        def on_response(response):
+            """Response callback handler"""
+
+            if hasattr(response, 'error') and response.error:
+                # We may had non HTTPErrors generated by tornado. (For
+                # example, when a host can't be resolved. For that
+                # cases, we create a wrapper arount response exception
+                # and throw that new Exception
+                if not isinstance(response.error, HTTPError):
+                    errcode = response.code
+                    message = repr(response.error)
+                    response.error = HTTPError(errcode, message, response)
+            # No error case
+            return callback(Response(response))
+
         resource = AsyncHTTPClient()
         response = resource.fetch(
             url_concat(request.uri, env["params"]),
-            lambda x: callback(Response(x)),
+            callback=on_response,
             method=request.verb,
             headers=request.headers,
             body=env.get("body"),
-            use_gzip = request.use_gzip,
-            progress_callback = request.progress_callback,
-            connect_timeout = request.connect_timeout,
-            request_timeout = request.request_timeout)
+            use_gzip=request.use_gzip,
+            ca_certs=request.ca_certs,
+            progress_callback=request.progress_callback,
+            connect_timeout=request.connect_timeout,
+            request_timeout=request.request_timeout)
+
+        # A cancellable 'future' ;)
         return response
 
     def execute(self, callback, chain, request, env):
@@ -156,7 +189,7 @@ class PayloadMarshallingProcessor(RequestProcessor):
     def execute(self, callback, chain, request, env):
         if env["payload"]:
             content_type = request.headers.get("content-type")
-            marshaller   = Converters.marshaller_for(content_type)
+            marshaller   = Converters.for_type(content_type)
             env["body"]  = marshaller.marshal(env["payload"])
             del(env["payload"])
         return chain.follow(callback, request, env)
@@ -201,7 +234,7 @@ tornado_chain = [
     AuthenticationSyncProcessor(),
 #   RedirectProcessor(),
     ExecuteRequestProcessor(),
-    ]
+]
 
 auth_chain = [
     PayloadMarshallingProcessor(),
