@@ -1,14 +1,12 @@
-#!/usr/bin/env python
-# -*- mode:python; tab-width: 2; coding: utf-8 -*-
+# -*- mode:python; coding: utf-8 -*-
 
 """
-request
+Request
 """
 
 from __future__ import absolute_import
 
-__author__ = "caelum - http://caelum.com.br"
-__modified_by__  = "Carlos Martin <cmartin@liberalia.net>"
+__author__  = "Carlos Martin <cmartin@liberalia.net>"
 __license__ = "See LICENSE.restfulie for details"
 
 # Import here any required modules.
@@ -18,6 +16,7 @@ __all__ = ['Request']
 # Project requirements
 from .parser import Parser
 
+
 class Request(object):
     """An HTTP request"""
 
@@ -30,9 +29,9 @@ class Request(object):
         the server
         """
         env = {
-            'params'  : params,
-            'payload' : body,
-            }
+            'params' : params,
+            'payload': body,
+        }
 
         procs = list(self._config.processors)
         return Parser(procs).follow(callback, self._config, env)
